/*
 * Copyright 2008, Systemic Realtime Design LLC. All Rights Reserved.
 *
 * This source code may contain confidential information of Systemic
 * Realtime Design LLC and its licensors. Any use, reproduction,
 * modification, disclosure, distribution or transfer of this software,
 * or any software which includes or is based upon any of this code, is
 * prohibited unless expressly authorized by SRD by written agreement. For
 * more information (including whether this source code file has been
 * published) please email licensing@sysrtime.com.
 */

/*
 * net/tipc/tipc_handler.c: TIPC signal handling
 * 
 * Copyright (c) 2000-2006, Ericsson AB
 * Copyright (c) 2005-2007, Wind River Systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the names of the copyright holders nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "tipc_core.h"
#include "tipc_dbg.h"
#include <pthread.h>
#include <process.h>
#include "quiesce.h"
#include "nw_thread.h"
#include "siglock.h"


/*
 * Declare public routines so "sparse" doesn't complain ...
 */
typedef struct {
	void (*func)(void*);
	void *arg;	
	struct callout timer;
} tipc_handler_t;

extern int  tipc_handler_start(void);
extern void tipc_handler_stop(void);

int tipc_handler_start(void)
{
	// nothing to do
	return EOK;
}


void tipc_handler_stop(void)
{
	// nothing to do
}


static void tipc_handler_thread (tipc_handler_t *handler)
{
	handler->func(handler->arg);
	callout_stop(&handler->timer);
	free(handler);
}


unsigned int tipc_k_signal(Handler routine, unsigned long argument)
{
#if 0 
	/* BUG: needs to spawn a new thread context, just calling the function leads to
	 *      mutex locking issues and potentially bug crashes from multiple free calls.
	 */
	routine (argument);
#else
	tipc_handler_t *handler = malloc(sizeof(tipc_handler_t));
	handler->func = (void*)routine;
	handler->arg  = (void*)argument;

	callout_init(&handler->timer);
	callout_msec(&handler->timer, 1, (void*)tipc_handler_thread, (void*)handler);
#endif
}


